/* 
 * inpowerd.c v1.0
 * Copyright 1999 by BALATON Zoltan <balaton@eik.bme.hu>
 * 
 * This program is a server which must be run from inetd. It signals init
 * about power events reported by an incoming connection. It implements
 * a primitive clear text password authentication (only the first 
 * [BUFFLEN-2] characters of the password is significant). For greater
 * security it is advised to use tcp_wrappers to restrict who can connect
 * to this service.
 */

#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <syslog.h>
#include "powerfail.h"

#define BUFFLEN  10
#define PWDFILE  "/etc/inpowerd.pwd"

static char *version = "in.powerd v1.0 Copyright 1999 by BALATON Zoltan\n";
char buf[BUFFLEN], pwd[BUFFLEN];

static int readstr(int fd, char *buf, size_t len);

int main(void)
{
  int fd, len, res;

  openlog("in.powerd", LOG_CONS | LOG_PID, LOG_DAEMON);

  if( (fd = open(PWDFILE, O_RDONLY)) < 0) {
    syslog(LOG_ERR, "Can't open %s which should contain a password to verify incoming connections. Reason: %m\n", PWDFILE);
    exit(1);
  }
  if( (len = readstr(fd, pwd, BUFFLEN-1)) <= 0) {
    syslog(LOG_ERR, "Can't read password from %s.", PWDFILE);
    if( len < 0 )
      syslog(LOG_ERR, "Reason: %m\n");
    exit(1);
  }
  close(fd);

  if( (res = readstr(STDIN_FILENO, buf, BUFFLEN)) <= 0) {
    syslog(LOG_ERR, "Can't read data from peer.");
    if( len < 0 )
      syslog(LOG_ERR, "Reason: %m\n");
    exit(1);
  }

  if( strcmp(pwd, buf+1) ) {
    syslog(LOG_ERR, "Authentication failed: bad password.\n");
    exit(1);
  }

  if( buf[0] != 'F' && buf[0] != 'L' && buf[0] != 'O' ) {
    syslog(LOG_ERR, "Invalid power event received.\n");
    exit(1);
  }

  syslog(LOG_NOTICE, "Got power event %c, signalling init.\n", buf[0]);
  powerfail(buf[0]);
  exit(0);
}

/* Reads atmost <len-1> chars into <buf> from file descriptor <fd> and
 * forms a string by \0 terminating it at the first newline or after
 * <len-1> chars */
/* Returns the length of the string or -1 on error */
static int readstr(int fd, char *buf, size_t len)
{
  int res;
  char *p;

  if( (res = read(fd, buf, len-1)) > 0) {
    if( (p = strchr(buf, '\n')) != NULL )
      res = p - buf;
    buf[res] = '\0';
  }
  return(res);
}
