/*
 * powerfail.c	Notify init process about power status. Originally part of
 *		powerd in the SysVinit package.
 *
 * Author:	Miquel van Smoorenburg, <miquels@drinkel.cistron.nl>.
 *
 * Version:	1.31,  29-Feb-1996.
 *
 *		This program was originally written for my employer,
 *			** Cistron Electronics **
 *		who has given kind permission to release this program
 *		for general puppose.
 *
 *		Copyright 1991-1996 Cistron Electronics.
 *
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 * 
 *  Modified by BALATON Zoltan <balaton@eik.bme.hu> on 8-Mar-1999
 */

/* Use the new way of communicating with init. */
#define NEWINIT
/* COMPAT: SIGPWR reason (OK/FAIL/LOW) */
#define PWRSTAT  "/etc/powerstatus"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#ifdef NEWINIT
#include "initreq.h"
#endif

#ifndef SIGPWR
#  define SIGPWR SIGUSR1
#endif

#ifdef NEWINIT
static void alrm_handler(int unused)
{
}
#endif

/* Tell init about a power event. (<cmd>: O=OK, F=FAIL, L=LOW) */
void powerfail(char cmd)
{
  int fd;
#ifdef NEWINIT
  struct init_request req;

  /* Fill out the request struct. */
  memset(&req, 0, sizeof(req));
  req.magic = INIT_MAGIC;
  switch(cmd) {
    case 'F':
      req.cmd = INIT_CMD_POWERFAIL;
      break;
    case 'L':
      req.cmd = INIT_CMD_POWERFAILNOW;
      break;
    case 'O':
      req.cmd = INIT_CMD_POWEROK;
      break;
    default:
      return;
  }
  /* Open the fifo (with timeout) */
  signal(SIGALRM, alrm_handler);
  alarm(3);
  if ((fd = open(INIT_FIFO, O_WRONLY)) >= 0
		&& write(fd, &req, sizeof(req)) == sizeof(req)) {
	close(fd);
	return;
  }
  /* Fall through to the old method.. */
#endif

  /* Create an info file for init. */
  unlink(PWRSTAT);
  if ((fd = open(PWRSTAT, O_CREAT|O_WRONLY, 0644)) >= 0) {
    switch(cmd) {
      case 'F':
        write(fd, "FAIL\n", 5);
        break;
      case 'L':
        write(fd, "LOW\n", 4);
        break;
      case 'O':
       	write(fd, "OK\n", 3);
        break;
      default:
        close(fd);
        return;
    }
    close(fd);
  }
  kill(1, SIGPWR);
}
