/*
 * rpwrstat.c v1.0
 * Copyright 1999 by BALATON Zoltan <balaton@eik.bme.hu>
 * 
 * This program is a client for in.powerd. It signals in.powerd running
 * on another host about power events.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#ifdef N_PLAT_NLM
#include <errno.h>
#include <nwconio.h>
#define perror(x)  printf("%s: %s\n", (x), strerror(errno))
#define printf  ConsolePrintf
#endif

static char *prgname = "rpwrstat";
static char *version = "1.0";
static char *copyright = "Copyright 1999 by BALATON Zoltan";

void usage(void);

int main(int argc, char **argv)
{
  char *buf;
  int len, fd;
  struct sockaddr_in saddr;
  int r, res;

  printf("%s v%s %s\n", prgname, version, copyright);
  if( argc != 5 ) {
    printf("\n%s: Invalid number of parameters specified.\n", prgname);
    usage();
    exit(1);
  }
  if( *argv[3] != 'F' && *argv[3] != 'L' && *argv[3] != 'O' ) {
    printf("\n%s: Invalid power event specified.\n", prgname);
    usage();
    exit(1);
  }

  memset(&saddr, 0, sizeof(saddr));
  saddr.sin_family = AF_INET;
  if( (saddr.sin_addr.s_addr = inet_addr(argv[1])) == -1 ) {
    printf("\n%s: Invalid server address specified (only numerical addresses permitted).\n", prgname);
    exit(1);
  }
  saddr.sin_port = atoi(argv[2]);
  if( saddr.sin_port == 0 ) {
    printf("\n%s: Invalid port specified (only numerical values permitted).\n", prgname);
    exit(1);
  }
  saddr.sin_port = htons(saddr.sin_port);

  len = strlen(argv[4])+2;
  if( (buf = malloc(len)) == NULL ) {
    printf("\n%s: Cannot allocate memory.\n", prgname);
    exit(1);
  }
  *buf = *argv[3];
  strcpy(buf+1, argv[4]);

  if( (fd = socket(PF_INET, SOCK_STREAM, 0)) >= 0)
    if( (res = connect(fd, (struct sockaddr *)&saddr, sizeof(saddr))) == 0 )
      for(r=0; r<len; r+=res) {
        res = write(fd, buf+r, len-r);
        if( res < 0 ) break;
      }

  if( fd < 0 ) {
    printf("\n%s: Cannot create socket.\n", prgname);
    res = 1;
  } else if( res < 0 ) {
    printf("\n%s: Error sending data.\n", prgname);
    res = 1;
  } else {
    res = 0;
  }
  if( res == 1 )
    perror(prgname);
  else
    printf("%s: Power event notification sent to %s about power event %c.\n",
           prgname, argv[1], *argv[3]);
  if( fd >= 0 ) close(fd);
  free(buf);
  exit(res);
}

void usage(void)
{
  printf("\nUsage: %s <server> <port> <event> <password>\n", prgname);
  printf("   <server> and <port> is the ip address and port of the server to connect to\n");
  printf("   <event> is the power event to report (O = OK, F = FAIL, L = LOW BATTERY)\n");
  printf("   <password> is the password used to authenticate us to the server\n");
}
